'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (header, props, event) {

  event.preventDefault();

  var headerNode = header.getDOMNode();
  var headerRegion = _region2.default.from(headerNode);
  var dragColumn = column;
  var dragColumnIndex;
  var columnData;
  var shiftRegion;

  (0, _dragHelper2.default)(event, {

    constrainTo: headerRegion.expand({ top: true, bottom: true }),

    onDragStart: function onDragStart(event, config) {

      var columnHeaders = headerNode.querySelectorAll('.' + props.cellClassName);

      columnData = props.columns.map(function (column, i) {
        var region = _region2.default.from(columnHeaders[i]);

        if (column === dragColumn) {
          dragColumnIndex = i;
          shiftRegion = region.clone();
        }

        return {
          column: column,
          index: i,
          region: region
        };
      });

      header.setState({
        dragColumn: column,
        dragging: true
      });

      config.columnData = columnData;
    },
    onDrag: function onDrag(event, config) {
      var diff = config.diff.left;
      var directionSign = diff < 0 ? -1 : 1;
      var state = {
        dragColumnIndex: dragColumnIndex,
        dragColumn: dragColumn,
        dragLeft: diff,
        dropIndex: null,
        shiftIndexes: null,
        shiftSize: null
      };

      var shift;
      var shiftSize;
      var newLeft = shiftRegion.left + diff;
      var newRight = newLeft + shiftRegion.width;
      var shiftZone = { left: newLeft, right: newRight };

      config.columnData.forEach(function (columnData, index, arr) {

        var itColumn = columnData.column;
        var itRegion = columnData.region;

        if (shift || itColumn === dragColumn) {
          return;
        }

        var itLeft = itRegion.left;
        var itRight = itRegion.right;
        var itZone = directionSign == -1 ? { left: itLeft, right: itLeft + itRegion.width } : { left: itRight - itRegion.width, right: itRight };

        if (shiftRegion.width < itRegion.width) {
          // shift region is smaller than itRegion
          shift = _region2.default.getIntersectionWidth(itZone, shiftZone) >= Math.min(itRegion.width, shiftRegion.width) / 2;
        } else {
          // shift region is bigger than itRegion
          shift = _region2.default.getIntersectionWidth(itRegion, shiftZone) >= itRegion.width / 2;
        }

        if (shift) {
          shiftSize = -directionSign * shiftRegion.width;
          state.dropIndex = index;
          state.shiftIndexes = buildIndexes(directionSign, index, dragColumnIndex);
          state.shiftSize = shiftSize;
        }
      });

      header.setState(state);
    },
    onDrop: function onDrop(e) {
      header.onDrop(e);
    }
  });
};

var _region = require('region');

var _region2 = _interopRequireDefault(_region);

var _dragHelper = require('drag-helper');

var _dragHelper2 = _interopRequireDefault(_dragHelper);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function range(start, end) {
  var res = [];

  for (; start <= end; start++) {
    res.push(start);
  }

  return res;
}

function buildIndexes(direction, index, dragIndex) {
  var indexes = direction < 0 ? range(index, dragIndex) : range(dragIndex, index);

  var result = {};

  indexes.forEach(function (value) {
    result[value] = true;
  });

  return result;
}